#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <pthread.h>

#define checkError(expr,userMsg)\
    do {\
        if (!(expr)) {\
            perror(userMsg);\
            exit(EXIT_FAILURE);\
        }\
    } while(0)
#define checkErrorPthread(err,userMsg) \
    do {\
        int _err = err;\
        if (_err > 0) { \
            errno = _err;\
            checkError(0,userMsg);\
        } \
    } while(0);

#define MAX_SIZE (1024)

int arraySize = 0;
int globalArray[MAX_SIZE];

pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t cond = PTHREAD_COND_INITIALIZER;

typedef struct {
    int id;
} inData_t;

void* threadFunc(void* arg) {

    inData_t* data = (inData_t*) arg;

    while (1) {

        pthread_mutex_lock(&lock);
        while (arraySize == 0) {
            pthread_cond_wait(&cond, &lock);
        }

        printf("Aktivirala se nit: %d\n", data->id);

        if (arraySize == -1) {
            pthread_mutex_unlock(&lock);
            break;
        }

        int sp = 0;
        for (int i = 0; i < arraySize; i++) {
            sp += globalArray[i]*globalArray[i];
        }
        arraySize = 0;
        printf("Skalarni proizvod: %d\n", sp);

        pthread_mutex_unlock(&lock);
    }

    printf("Zavrsila se nit: %d\n", data->id);

    return NULL;
}

int main(int argc, char** argv) {

    checkError(argc == 2, "args");

    int n = atoi(argv[1]);
    pthread_t* tids = malloc(n*sizeof(pthread_t));
    checkError(tids != NULL, "tids");

    inData_t* data = malloc(n*sizeof(inData_t));
    checkError(data != NULL, "data");

    for (int i = 0; i < n; i++) {
        data[i].id = i + 1;
        pthread_create(&tids[i], NULL, threadFunc, &data[i]);
    }

    while (1) {

        pthread_mutex_lock(&lock);
        if (arraySize > 0) {
            pthread_mutex_unlock(&lock);
            continue;
        }

        scanf("%d", &arraySize);

        if (arraySize == -1) {
            pthread_mutex_unlock(&lock);
            pthread_cond_broadcast(&cond);
            break;
        }

        for (int i = 0; i < arraySize; i++) {
            scanf("%d", &globalArray[i]);
        }

        pthread_mutex_unlock(&lock);
        pthread_cond_signal(&cond);
    }


    for (int i = 0; i < n; i++) {
        pthread_join(tids[i], NULL);
    }

    free(tids);
    free(data);

    exit(EXIT_SUCCESS);
}