#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <pthread.h>
#include <time.h>
#include <signal.h>

#define checkError(expr,userMsg)\
    do {\
        if (!(expr)) {\
            perror(userMsg);\
            exit(EXIT_FAILURE);\
        }\
    } while(0)
#define checkErrorPthread(err,userMsg) \
    do {\
        int _err = err;\
        if (_err > 0) { \
            errno = _err;\
            checkError(0,userMsg);\
        } \
    } while(0);

int sigusr1 = 0;
int sigusr2 = 0;
int sigterm = 0;

void sigHandler(int signum) {
    switch(signum) {
        case SIGUSR1:
            sigusr1 = 1;
            break;
        case SIGUSR2:
            sigusr2 = 1;
            break;
        case SIGTERM:
            sigterm = 1;
            break;
    }
}

int main(int argc, char** argv) {

    
    int x;
    do {
        checkError(signal(SIGUSR1, sigHandler) != SIG_ERR, "sig");
        checkError(signal(SIGUSR2, sigHandler) != SIG_ERR, "sig");
        checkError(signal(SIGTERM, sigHandler) != SIG_ERR, "sig");

        pause();
        if (sigterm) {
            break;
        }

        scanf("%d", &x);

        if (sigusr1) {
            printf("%d\n", x < 0 ? -x : x);
            sigusr1 = 0;
        }
        if (sigusr2) {
            printf("%d\n", x*x);
            sigusr2 = 0;
        }
    } while (!sigterm);

    exit(EXIT_SUCCESS);
}