#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <pthread.h>
#include <time.h>
#include <math.h>

#define checkError(expr,userMsg)\
    do {\
        if (!(expr)) {\
            perror(userMsg);\
            exit(EXIT_FAILURE);\
        }\
    } while(0)
#define checkErrorPthread(err,userMsg) \
    do {\
        int _err = err;\
        if (_err > 0) { \
            errno = _err;\
            checkError(0,userMsg);\
        } \
    } while(0);

typedef struct {
    int start;
} inData_t;

typedef struct {
    float norma;
} outData_t;

int m, n, k;
float** vektori;

void* threadFunc(void* arg) {

    inData_t* data = (inData_t*)arg;

    float maksNorma;
    int init = 0;
    for (int i = 0; i < m/k; i++) {
        float norma = 0;
        for (int j = 0; j < n; j++) {
            norma += vektori[i+data->start][j]*vektori[i+data->start][j];
        }
        norma = sqrt(norma);
        if (init) {
            if (norma > maksNorma) {
                maksNorma = norma;
            }
        }
        else {
            maksNorma = norma;
            init = 1;
        }
    }

    outData_t* out = malloc(sizeof(outData_t));
    checkError(out != NULL, "...");
    out->norma = maksNorma;

    return out;
}

int main(int argc, char** argv) {

    scanf("%d%d%d", &m, &n, &k);
    vektori = malloc(m*sizeof(float*));
    checkError(vektori != NULL, "...");

    for (int i = 0; i < m; i++) {
        vektori[i] = malloc(n*sizeof(float));
        checkError(vektori[i] != NULL, "..");
        for (int j = 0; j < n; j++) {
            scanf("%f", &vektori[i][j]);
        }
    }

    pthread_t* tids = malloc(k*sizeof(pthread_t));
    checkError(tids!=NULL, "...");

    inData_t* data = malloc(k*sizeof(inData_t));
    checkError(data!=NULL, "...");

    for (int i = 0; i < k; i++) {
        data[i].start = i*(m/k);
        checkErrorPthread(
            pthread_create(&tids[i], NULL, threadFunc, &data[i]),
            "create"
        );
    }

    int redniBrNiti = -1;
    float maksNorma;
    for (int i = 0; i < k; i++) {
        outData_t* value = NULL;
        checkErrorPthread(
            pthread_join(tids[i], (void**)&value),
            "join"
        );
        if (i == 0) {
            redniBrNiti = 0;
            maksNorma = value->norma;
        }
        else {
            if (value->norma > maksNorma) {
                maksNorma = value->norma;
                redniBrNiti = i;
            }
        }
        free(value);
    }

    printf("%d %f\n", redniBrNiti, maksNorma);

    free(tids);
    free(data);
    for (int i = 0; i < m; i++) {
        free(vektori[i]);
    }
    free(vektori);

    exit(EXIT_SUCCESS);
}